import React, {useState, useCallback, useEffect} from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleBand, scaleLinear, max, format, extent  } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';


const width = 960;
const menuHeight = 60;
const height = 500 - menuHeight;
const margin = {top: 50, right:50, bottom:70, left: 100};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 50;
const tickOffset = 5;
const circleRadius = 7;

const attributes = [
  {value: "sepal_length", label: "Sepal Length"},
  {value: "sepal_width", label: "Sepal Width"},
  {value: "petal_length", label: "Petal Length"},
  {value: "petal_width", label: "Petal Width"},
  {value: "species", label: "Species"}
  ];

const getLabel = value => {
	for( let i = 0; i < attributes.length; i++) {
		if(attributes[i].value == value){
    	return attributes[i].label;
    }
	}
}

const App = () => {

  const data = useData();

  const initialxAttribute = 'petal_length'
  const [xAttribute, setXAttribute] = useState(initialxAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);
  
  const initialYAttribute = 'sepal_width'
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  
  if (!data) {
    return <pre>Loading...</pre>;
  }
    
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
	const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G','B')
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
  

	console.log("X Scale Ticks:", xScale.ticks(5));
 	console.log("Y Scale Ticks:", yScale.domain());
  return (
    <>
    <label for="x-select">X: </label>
      <Dropdown 
        options={attributes} 
        id="x-select" 
        selectedValue={xAttribute}
        onSelectedValueChange={setXAttribute}
        />
    <label for="y-select">Y: </label>
      <Dropdown 
        options={attributes} 
        id="y-select" 
        selectedValue={yAttribute}
        onSelectedValueChange={setYAttribute}
        />
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom 
          xScale={xScale} 
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
          />
        <text 
          className="axis-label" 
          x={innerWidth / 2} 
          y={innerHeight + xAxisLabelOffset} 
          textAnchor="middle">
          {xAxisLabel}
        </text>
        <AxisLeft 
          yScale={yScale} 
          innerWidth={innerWidth} 
          />
        <text 
          className="axis-label" 
          textAnchor="middle"
          transform={` translate(${-yAxisLabelOffset},${innerHeight / 2}),rotate(-90)`}
          >
          {yAxisLabel}
        </text>
        <Marks 
          data={data} 
          xScale={xScale} 
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={circleRadius}
          />
      </g>
    </svg>
      </>
  );
};


const rootElement = document.getElementById('root');
  
ReactDOM.render(<App />, rootElement);